
CatDPSTest = true
if(Tests == nil) then
	SkipTests=true
	require('test')
	SkipTests=false
end

require('CatDPS')


local Player = CatPoints.Modifiers['CatDPS']:new()


function Tests.ZeroHaste()
	local actual = Player:new({Haste=0}).AttackSpeed
	local expected = 1

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end


Tests.OneHaste = function()
	local actual = Player:new({Haste=15.76}).AttackSpeed
	local expected = .99

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end


Tests.Expertise = function()
	local actual = Player:new({Expertise = 4*3.92}).DodgeReduction
	local expected = .01

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.Melee = function()
		local base = Player:new({AttackPower=0,WeaponDamage=0})
		local plus14AP = Player:new({AttackPower=14, WeaponDamage=0})
		local plus1Weapon = Player:new({WeaponDamage=1, AttackPower=0})

		return TEST(plus14AP.Melee==plus1Weapon.Melee and plus14AP.Melee==(base.Melee+1),
			format("Expected melee values to match: %.2f, %.2f, %.2f", base.Melee, plus14AP.Melee, plus1Weapon.Melee))
	end

Tests.HitRating= function()
	local actual = Player:new({Hit = 15.77}).MissReduction
	local expected = .01

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.SavageFury = function()
	local expected = Player:new({SavageFury=0, WeaponDamage=0, AttackPower=0, Idol='None'}).MangleDamage*1.2
	local actual = Player:new({SavageFury=2, WeaponDamage=0, AttackPower=0, Idol='None'}).MangleDamage

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.Mangle = function()
	local expected = 2*1.03*1.1*(1.6+264+24)/40
	local player = Player:new({Diamond=true,Melee=1, Idol='IdoloftheWild', Ferocity=5, PredatoryInstincts=5,})

	local actual = player.MangleCritDamage/player.MangleCost

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.SavageFury = function()
	local expected = Player:new({            Idol='None', WeaponDamage=0, AttackPower=0}).MangleDamage*1.2
	local actual = Player:new({SavageFury=2, Idol='None', WeaponDamage=0, AttackPower=0}).MangleDamage

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.Shred = function()
	local expected = 2*(1.1)*1.03*((2.25+405)+88+75)/42
	local player = Player:new({Diamond=true, Melee=1, Nordrassil=4, Idol='EverbloomIdol', ShreddingAttacks=2, PredatoryInstincts=5})
	local actual = player.ShredCritDamage/player.ShredCost

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.Rip = function()
	local expected = 1.15*(1553 + .24 + 120)
	local player = Player:new({AttackPower=1,Idol='IdolofFeralShadows', Thunderheart=4})
	local actual = player.RipDamage(5)

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.HeartOfTheWild = function()
	local expected = 11
	local player = Player:new({HeartoftheWild=5})


	setglobals({})

	local after=player:applyBonus({ATTACKPOWERFERAL=5, ATTACKPOWER=5})
	local actual = after.AttackPower - player.AttackPower

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.Strength = function()
	local expected = 5
	local player = Player:new({HeartoftheWild=5, SurvivaloftheFittest=3})


	setglobals({})
	local after=player:applyBonus({STR=expected})

	--* sotf * ap per str * HOTW
	expected = expected * 1.03 * 2 *1.1
	local actual = after.AttackPower

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.AgiAP = function()
		local expected = 5
		local player = Player:new({HeartoftheWild=5, SurvivaloftheFittest=3})


		setglobals({})

		local after=player:applyBonus({AGI=expected})

		--* sotf * ap per str * HOTW
		expected = expected * 1.03 * 1 *1.1
		local actual = after.AttackPower

		local error = PercentError(actual, expected)
		return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
	end



Tests.Crit = function()
	local expected = 25
	local player = Player:new({SurvivaloftheFittest=3})

	setglobals({})

	after=player:applyBonus({AGI=expected, CR_CRIT=22.1})

	--* sotf * ap per agi * HOTW
	expected = (expected*1.03)/2500 + .01
	local actual = after.Crit - player.Crit

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.Equiv = function()
	local player=Player:new({SurvivaloftheFittest=3, Idol='None'})

	setglobals({Idol='None'})

	--local equiv = player:equivalence(true)
	return true, "\tPassed"
end

Tests.Idol = function()
	local expected = 'None'
	local player=Player:new({SurvivaloftheFittest=3})

	setglobals({})

	local p = player:applyBonus({Idol=expected})
	if(p.Idol==expected) then
		return true, "\tPassed"
	else
		return false, "\tFailed"
	end

end


Tests.Miss = function()
	local player = Player:new({Miss=0, Dodge=0})
	local expected = player.MangleCost

	local actual = player.MangleAmortizedCost
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end


Tests.MangleEnergy = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 16.6
	player:calculate()
	local actual = TraceStep.totalEnergyAfterMangle
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.Melee1 = function()
	setglobals({})

	local player = Player:new({Haste=50, SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 470.0362685049257
	player:calculate()
	local actual = TraceStep.meleeDmg1
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcMelee = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 243.42857142857142
	player:calculate()
	local actual = TraceStep.Melee
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcPI = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 5
	player:calculate()
	local actual = TraceStep.PredatoryInstincts
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcMiss = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = .05-player.Expertise*52/82/2.5*.25/100
	player:calculate()
	local actual = TraceStep.totalMiss
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcWhiteCrit = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = .3
	player:calculate()
	local actual = TraceStep.whiteCrit
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcTotalMangleDamage = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 1066.4886857142855

	player:calculate()
	local actual = TraceStep.totalMangleDamage
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcMangleDamage = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 784.1828571428571

	player:calculate()
	local actual = TraceStep.MangleDamage
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcAttacks = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 3.3790980491512545

	player:calculate()
	local actual = TraceStep.attacks
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcShredTime = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 6.61704578694737

	player:calculate()
	local actual = TraceStep.shredsSegment
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcShredAmortized = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 40.746146341463415

	player:calculate()
	local actual = TraceStep.ShredAmortizedCost
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcShredMelee = function()
	setglobals({})

	local player = Player:new({Haste=50, SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 2528.8710848328337

	player:calculate()
	local actual = TraceStep.meleeDmgAfterShreds
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcShredTotal = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 3794.950625142857

	player:calculate()
	local actual = TraceStep.totalShredDamage
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcCycleTime = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 14.942530551490305

	player:calculate()
	local actual = TraceStep.cycleTime
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcMeleeDmgBeforeRip = function()
	setglobals({})

	local player = Player:new({Haste=50,SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 4657.377650616198

	player:calculate()
	local actual = TraceStep.meleeDmgBeforeRip
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcRipDamage = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 2517.21717

	player:calculate()
	local actual = TraceStep.ripDamage
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcDPS = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	player.Enemy.Mitigation = 0.7016115633826216
	local expected = 675.77872883927

	local actual = player:calculate()
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end


Tests.CalcFinalMangles = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 1066.4886857142855
	player:calculate()
	local actual = TraceStep.finaltotalMangleDamage
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end

Tests.CalcFinalShreds = function()
	setglobals({})

	local player = Player:new({SavageFury=2, Ferocity=5, PredatoryInstincts=5, ShreddingAttacks=2, Naturalist=5, AttackPower=2500, Hit=142, Crit=.3, WeaponDamage=10, Expertise=20})
	local expected = 3794.950625142857
	player:calculate()
	local actual = TraceStep.finaltotalShredDamage
	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
end


if(not SkipTests) then
	RunTests(--Tests
	{Malorne4=Tests.Malorne4}
	, false, true)
end